"""
Create detached PDS label from CIDA non-ancillary event data table
  data attached PDS label

- Assumes data are TABLEs
- Assumes labels have no CONTAINERs, and one ^STRUCTURE target per TABLE
  that contains only COLUMN OBJECTs.

Usage (BASH):

  for t in tempel1 isp nonsci ; do

    ls data/$t/t*.tab \
    | python cidaeventtablbl.py extras/detached/$t

    cp data/$t/t*.tab  extras/detached/$t/

  done

Alternate (for SDC):

  cd .../extras/detached/

  for rt in "9P tempel1" "INTERSTELL isp" "NON.SCI nonsci" ; do

    toks=( $rt )
    r=${toks[0]}
    t=${toks[1]}

    grep -l "^ *TARGET_NAME *= \"*$r" ../../latest_jouni/t*.tab \
    | while read tab ; do cp $tab ./$t/ ; echo $tab ; done \
    | python cidaeventtablbl.py ./$t

  done
"""
import os
import re
import sys

### Destination directory is first, only, and required argument

dstdir=sys.argv[1]

### Create destination directory if it does not exist

if not os.path.isdir(dstdir): os.makedirs(dstdir)

### Regular expressions to identify certain ODL statements

rgxEND=re.compile('^ *END *\r?$')
rgxOBJTBL=re.compile('^ *OBJECT *= *[A-Z_]+_TABLE *\r?$')
rgxENDOBJ=re.compile('^ *END_OBJECT *= *[A-Z_]+_TABLE *\r?$')
rgxSTRUCT=re.compile('^ *\^STRUCTURE *= *"[A-Z0-9_.]*" *\r?$')
rgxPTR1=re.compile('^ *\^[A-Z0-9]*_TABLE *= *[0-9]* *<BYTES> *\r?$')

### Initialize header length; will be determined from first data pointer

hdrLen=0

########################################################################
### Read each line from stdin which contains the name of the CIDA
### data file to convert

for linFn in sys.stdin.readlines():

  ### Strip input to get path to CIDA data
  ### Determine directory name for ^STRUCTURE pointers to .FMT files
  ### Skip if path does not end in .tab or .TAB

  tblFn=linFn.strip(' \r\n')
  if tblFn[-4:].lower()!='.tab': continue
  tblDn=os.path.dirname(tblFn)
  tblBn=os.path.basename( tblFn[:-4] )

  ### Open table file for read, open detached label file for write

  fTbl = open( tblFn, "rb")
  fLbl = open( os.path.join(dstdir,tblBn+'.lbl'), "wb" )

  ######################################################################
  ### Initialize inTbl boolean to keep track of when reading TABLE OBJECT
  ### Read-table-file loop:  read a line at a time from the table file

  inTbl=False

  while True:

    odllin = fTbl.readline()

    ### If currently reading TABLE object:

    if inTbl:

      ### TABLE:  if end of TABLE object, reset boolean, drop through
      ###         to copy line to output

      if rgxENDOBJ.match(odllin):
        inTbl=False

      ### TABLE:  if ^STRUCTURE pointer, copy target to to output,
      ###         do not drop through (do not copy ^STRUCTURE to output

      elif rgxSTRUCT.match(odllin):
        fLbl.write( "\r\n        /* START OF TABLE COLUMNS */\r\n" \
                    + odllin.replace('^','/* ^'
                           ).replace('          \r','\r'
                           ).replace('\r',' */\r'
                           ) )
        fLbl.write( open(os.path.join(tblDn,odllin.split('"')[1].lower())
                        ).read().replace('OBJECT        ','        OBJECT'
                               ).replace('END_        OBJECT'
                                        ,'        END_OBJECT') )
        fLbl.write( "        /* END OF TABLE COLUMNS */\r\n\r\n" )
        continue

      ### TABLE:  Any other TABLE OBJECT line drops through

    ### If this is an END statement, output HEADER info and break out of
    ### the read-table-file loop

    if rgxEND.match(odllin):
      fLbl.write( """\r
^HEADER             = 1<BYTES>\r
OBJECT              = HEADER\r
        BYTES       = %d\r
        HEADER_TYPE = "ODL"\r
        DESCRIPTION = "Original attached PDS label for CIDA Event TABLE"\r
END_OBJECT          = HEADER\r
\r
END\r
""" % (hdrLen-1) )
      break

    ### If this is the first TABLE pointer, save the BYTES number for the
    ### header length

    if rgxPTR1.match(odllin):
      toks = odllin.strip(' \r\n' ).split('=')
      if hdrLen==0:
        hdrLen = eval( toks[1].split('<')[0] )
      toks.insert( 1, '"%s.TBL"' % (tblBn.upper(),) )
      toks[0]=toks[0].strip()
      odllin = '%s = ( %s, %s )\r\n' % tuple(toks)

    ### If this is a OBJECT = *_TABLE line, set in-TABLE boolean

    if rgxOBJTBL.match(odllin): inTbl=True

    ### Write line to output if we dropped through to here
    fLbl.write( odllin )

    ### End of while True: read-table-file loop
    ####################################################################

  ### Close files

  fLbl.close()
  fTbl.close()

  ### End of for linFn in sys.stdin.readlines(): loop reading input
  ### table filenames
  ######################################################################
