"""
Create detached PDS label from CIDA ancillary data table attached PDS label

- Assumes data are TABLEs
- Handles CONTAINER OBJECTs
- Assumes ^STRUCTURE target contains only COLUMN OBJECTs.

Usage (BASH):

  for tab in data/ancillary/*.tab ; do
    python pdsdecontain.py $tab > extras/detached/ancillary/`basename $tab`
  done

Alternate (SDC):

  cd .../extras/detached
  mkdir -p ancillary

  for tab in ../../latest_jouni/[^t]*.tab ; do
    python pdsdecontain.py $tab > ancillary/`basename ${tab%.tab}.lbl`
    cp $tab ancillary/`basename ${tab%.tab}.tbl`
  done
"""
import re
import os
import sys

########################################################################
### Regular expressions to identify certain ODL statements

rgxEND=re.compile('^ *END *\r?$').match

rgxOBJCON=re.compile('^ *OBJECT *= *CONTAINER *\r?$').match
rgxCONVAL=re.compile(
  '^ *(START_BYTE|BYTES|REPETITIONS) *= *[1-9][0-9]* *\r$'
).match
rgxENDOBJCON=re.compile('^ *END_OBJECT *= *CONTAINER *\r?$').match

rgxOBJCOL=re.compile('^ *OBJECT *= *COLUMN *\r?$').match
rgxCOLSTART_BYTE=re.compile( '^ *START_BYTE *= *[1-9][0-9]* *\r$').match
rgxCOLNUM=re.compile( '^ *COLUMN_NUMBER *= *[1-9][0-9]* *\r$').match
rgxENDOBJCOL=re.compile('^ *END_OBJECT *= *COLUMN *\r?$').match

rgxOBJ=re.compile('^ *OBJECT *= *[A-Z_]+ *\r?$').match
rgxENDOBJ=re.compile('^ *END_OBJECT *= *[A-Z_]+ *\r?$').match

rgxSTRUCT=re.compile('^ *\^STRUCTURE *= *"[A-Z0-9_.]*" *\r?$').match

rgxTBLPTR=re.compile(
  '^ *\^[A-Z_]*TABLE *= *[1-9][0-9]* *[<BYTES>]* *\r?$'
).match

########################################################################
### Routine to handle ^STRUCTURE = 'XYZ.FMT'
### - Actual filename is lowercase i.e. xyz.fmt
### - called by container() or notContainer() when ^STRUCTURE found
### - pass it to notContainer() until notContainer() returns it closed

def structure( lin, byteOff=0, colOff=0):
  sys.stdout.write( '\r\n' )
  exec( lin.strip('^ \r\n') )
  STRUCTURE = os.path.join( os.path.dirname(sys.argv[1]), STRUCTURE.lower() )
  fNew = open( STRUCTURE, 'rb')
  while not fNew.closed:
    colOff = notContainer( fNew, byteOff=byteOff, colOff=colOff )
  return colOff


########################################################################
### Routine to handle OBJECT = CONTAINER
### - Called by notContainer() when ^STRUCTURE found
### - Read CONTAINER keywords, hand info back to notContainer() when
###   sub-OBJECT or ^STRUCTURE found

def container( f
             , byteOff=0
             , colOff=0
             ):
  conLevel=0

  while True:
    lastTell = f.tell()
    lin = f.readline()

    if lin=='':
      f.close()
      return colOff

    if rgxOBJCON(lin) and conLevel==0:
      conLevel=1
      continue

    if rgxENDOBJCON(lin):
      return colOff

    if rgxOBJ(lin):
      f.seek(lastTell,0)
      conOff = notContainer( byteOff, colOff)
      continue

    if rgxSTRUCT(lin): colOff = structure( lin, byteOff=byteOff, colOff=colOff)

  ### End while True:

  return colOff

########################################################################
### Routine to handle everything else
### - copies lines verbatim except for CONTAINER OBJECT lines
### - copies first complete OBJECT including contained OBJECTs
### - hands any CONTAINER OBJECT off to container() method above
###   - reads them first to assess info
### - updates and returns column offset
###   - return -1 if END statement found
### Arguments:
###   f       - open file descriptor
###   byteOff - byte offset for current OBJECT (TABLE, CONTAINER)
###   colOff  - column offset in current TABLE OBJECT

def notContainer( f
                , byteOff=0
                , colOff=0
                ):

  ### Initialize object level maintains OBJECT depth,
  ###   return when it decrements to zero

  objLevel = 0

  ######################################################################
  ### Loop through lines:

  while True:

    ### Keep track of position at start of each line read

    lastTell = f.tell()
    lin = f.readline()

    ### If EOF, close file and return (reading ^STRUCTURE target)

    if lin=='':
      f.close();
      return colOff

    ####################################################################
    ### If OBJECT = CONTAINER, don't output lines, but ...

    if rgxOBJCON(lin):

      ### ... read CONTAINER OBJECT plus included CONTAINERs
      ###     - Don't print any CONTAINER info out (continue at end of if)
      ###     - conLevel maintains container levels; initial CONTAINER is 1
      ###     - final END_OBJECT = CONTAINER will
      ###       reset conLevel to zero and exit loop

      conLevel=1
      while conLevel>0:
        lin = f.readline()
        if lin=='': break

        ### ... increment or decrement CONTAINER level

        if rgxOBJCON(lin): conLevel += 1; continue
        if rgxENDOBJCON(lin): conLevel -= 1; continue

        ### ... read important keywords for CONTAINER level 1

        if conLevel==1 and rgxCONVAL(lin): exec(lin.strip('\n\r '))

      ### ... CONTAINER complete; save position at end of CONTAINER

      conEnd = f.tell()

      ### ... for CONTAINER REPETITIONS, pass CONTAINER to container()
      ###     method above, which will pass any lines not from this
      ###     CONTAINER OBJECT back to this method (even if they are
      ###     another CONTAINER OBJECT

      for i in range(REPETITIONS):
        f.seek( lastTell, 0 )
        colOff = container( f
                          , byteOff=byteOff + (START_BYTE-1) + i*BYTES
                          , colOff=colOff
                          )

      ### ... Reset read position to end of CONTAINER
      ### ... If objLevel is 0, CONTAINER was first object found, so return
      ### ... Otherwise continue to next line

      f.seek( conEnd, 0 )
      if objLevel==0: return colOff
      continue

      ### End of CONTAINER OBJECT if clause
      ##################################################################

    ####################################################################
    ### If ^STRUCTURE pointer, hand it off to structure() method, which
    ### will hand the target lines back to this routine.  Then skip to
    ### next line (don't re-write ^STRUCTURE line)

    if rgxSTRUCT(lin):
      colOff = structure( lin, byteOff=byteOff, colOff=colOff)
      continue

    ####################################################################
    ### Process certain COLUMN OBJECT keywords
    ### - correct START_BYTE per byteOff
    ### - correct COLUMN_NUMBER per colOff
    ### - increment colOff at END_OBJECT = COLUMN
    ### - Insert filename into ^*_TABLE = N becomes ( "XYZ.TBL" ,N)

    if rgxCOLSTART_BYTE(lin):
      exec( lin.strip('\n\r ') )
      lin = lin.rstrip(' 1234567890\r\n') + ' %d\r\n' % (START_BYTE+byteOff,)
    elif rgxCOLNUM(lin):
      lin = lin.rstrip(' 1234567890\r\n') + ' %d\r\n' % (colOff+1,)
    elif rgxOBJCOL(lin):
      lin=lin.replace('OBJECT          ','        OBJECT')
    elif rgxENDOBJCOL(lin):
      lin=lin.replace('END_OBJECT          ','        END_OBJECT')
      colOff += 1
    elif rgxTBLPTR(lin):
      lin=lin.replace(' ','').replace(
      '=',' = ( "%s", ' % os.path.basename(sys.argv[1][:-4]+'.tbl')
      ).upper().rstrip(' \r\n')+' )\r\n'

    ####################################################################
    ### Copy input line, possibly modified, to stdout

    sys.stdout.write(lin)

    ####################################################################
    ### Return colOff of -1 if END statument found

    if rgxEND(lin): return -1

    ####################################################################
    ### Track OBJECT level objLevel, return when objLevl decrements to 0

    if rgxOBJ(lin):
      objLevel += 1

    if rgxENDOBJ(lin):
      objLevel -= 1
      if objLevel==0: return colOff

  ### End While True loop which reads line from file descriptor


########################################################################
########################################################################
### Main routine:
###   - Open file using first, only and required argument as filename,
###     pass file descriptor to nonContainer() method repeated until -1
###     is returned indicating end of file, then close file
###   - notContainer(), container(), and structure() call each other
###     recursively to process the file along with containers and any
###     files indicated via ^STRUCTURE pointers.

f = open(sys.argv[1],'rb')
rtn = 0
while rtn>-1: rtn = notContainer( f )
f.close()

### End
########################################################################
